# plASgraph2 with extremities and homology

Following document consists of 5 main parts:
1. List of all files with summarized changes compared to original plASgraph files
2. Tutorials for different versions
3. Data availability
4. Results summarization
5. Tips

## List of files with short description with summarized changes

Small to no important changes:
* src/train.py - no change
* src/test.py - no change
* src/create_graph.py - no change
* src/architecture.py - no change
* src/summarize_gold_standard.py - no change
* src/eval.py - different version of eval than is used in final plASgraph2. 
  Only added part for accuracy computation.
* src/thresholds.py - no change
* src/thresholds_extremities.py - small changes for 2n nodes
* src/homology_scoring_train.py - set log odd score to zero if pseudo count is 0
* src/summarize_features.py - no change
* src/config.py - add paramter for extremity version and method for adding new features to existing config
* src/helpers.py - add methods for extremities

Bigger changes:
* src/train_extremities.py - adjust for 2n nodes, based on `train.py`
* src/test_extremities.py - adjust for 2n nodes, based on `test.py`
* src/create_graph_extremities.py - adjust original graph for 2n nodes and add extremity graph, based on `create_graph.py`
* src/architecture_extremities.py -  add extremity versions 
* src/homology_scoring.py -- add params `--vectors`,`--binary_vectors` and add parts for saving vectors for each contig
* src/homology_scoring_lib.py -- added methods `extract_top_n_domains`, `choose_domains_with_query` and 
  `choose_domains_for_all_labels` for choosing domains


New files:
* src/generate_input.py - generate synthetic assembly graphs
* src/helpers_spektral.py - new class for storing two adjacency matrices. Class is based on Spektral class SingleLoader
* src/homology_choose_domains.py - script for choosing protein domains
* src/homology_fisher.py - calculate fisher exact test
* src/summarize_differences.py - script for summarizing differences of contigs from testing data between two experiments 
  - output tables are made with pandas describe() method and boxplots with seaborn catplot

# Tutorials

Common steps for all versions:
1. Create virtual environment 
    ```
    python3 -m venv plasgraph-venv
    source plasgraph-venv/bin/activate
    pip3 install pip3 install networkx pandas numpy scipy scikit-learn biopython matplotlib tensorflow spektral seaborn pyyaml statsmodels
    ```

2. Prepare folder for your experiment, for example `experiment1`

3. Prepare data and parameters. In these tutorials we will use following:
   1. csv file with train samples - `../sets/train_samples.csv`
   2. csv file with test samples - `../sets/test_samples.csv`
   3. path to directory with samples - `../data/`

4. Prepare config file, in these tutorials we will use `config.yaml`. Example of config you can find in `confiy.yaml`

## List of commands for basic program

1. Execute following commands:

```
# training
python3 ../src/train.py config.yaml ../sets/train_samples.csv ../data/ train-model -l train-model-log > train.log 2> train.err
# testing
python3 ../src/test.py set ../sets/test_samples.csv ../data/ train-model test-pred.csv > test.log 2> test.err
# prepare correct answers
python3 ../src/summarize_gold_standard.py ../sets/test_samples.csv ../data/ test-gold.csv
# evaluation
python3 ../src/eval.py -n pred test-gold.csv test-pred.csv > test-stats.csv
```


## Tutorial for Extremities

1. Specify which extremity version you want to use in config file. You can choose from 'v1', 'v1-two_gcn', 'v2', 'v2-two_dense', 'v2-two_gcn' and 'v2-two_gcn_two_dense' 
2. Execute following commands:

```
# training
python3 ../src/train_extremities.py config_extremities.yaml ../sets/train_samples.csv ../data/ train-model -l train-model-log > train.log 2> train.err
# testing
python3 ../src/test_extremities.py set ../sets/test_samples.csv ../data/ train-model test-pred.csv > test.log 2> test.err
# prepare correct answers
python3 ../src/summarize_gold_standard.py ../sets/test_samples.csv ../data/ test-gold.csv
# evaluation
python3 ../src/eval.py -n pred test-gold.csv test-pred.csv > test-stats.csv
```


## Tutorial for homology
1. Prepare file with homology features, for example `Pfam-scores-vectors.csv` and `test-Pfam-scores-vectors.csv`. Check section 
   Data preparation for homology
2. Make sure your config file contains all features you want to use
3. Execute following commands:

```
# training
python3 ../src/train.py config-log-odd-100.yaml ../sets/train_samples.csv ../data/ train-model -l train-model-log -f Pfam-scores-vectors.csv -l train-model-log > train.log 2> train.err
# testing
python3 ../src/test.py set ../sets/test_samples.csv ../data/ train-model test-pred.csv -f test-Pfam-scores-vectors.csv  > test.log 2> test.err
# prepare correct answers
python3 ../src/summarize_gold_standard.py ../sets/test_samples.csv ../data/ test-gold.csv
# evaluation
python3 ../src/eval.py -n pred test-gold.csv test-pred.csv > test-stats.csv
```

You can use homology on its own (use `train.py` and `test.py`) or you can use it with extremities (use 
`train_extremities.py` and `test_extremities.py`). Important is param `-f`.


## Data preparation for homology

Requirements: You need to have output from BLAST tool or use examples from data directory.

Steps:
1. Run `homology_scoring_train.py`
   1. With pseudo counts for log-odd score
       ```  
       python3 ../src/homology_scoring_train.py -p 10 ../sets/train_samples.csv ../data/ Pfam Pfam-model.tsv > homology_scoring_train.log 2> homology_scoring_train.err;
       ```
   2. Without pseudo counts for fisher exact test
       ```  
       python3 ../src/homology_scoring_train.py -p 0 ../sets/train_samples.csv ../data/ Pfam Pfam-model-no-pseudocount.tsv > homology_scoring_train-no-pseudocount.log;
       ```
2. Run `homology_scoring.py`. Specify if you want to create vectors with `--vectors` or `--binary_vectors`.
   ```
   python3 ../src/homology_scoring.py ../sets/train_samples.csv ../data/ Pfam Pfam-model.tsv Pfam-scores-vectors.csv --binary_vectors > homology_scoring.log 2> homology_scoring.err 
   python3 ../src/homology_scoring.py ../sets/test_samples.csv ../data/ Pfam Pfam-model.tsv test-Pfam-scores-vectors.csv --binary_vectors > homology_scoring-test_samples.log 2> homology_scoring-test_samples.err
   ```
3. Run script for computing fisher score if you want to choose domains based on it.
   ```
   python3 ../src/homology_fisher.py Pfam-model.tsv -o Pfam-model-no-pseudocount-fisher.tsv > fisher.log
   ```
4. Run `homology_choose_domains.py`. This script also prepares config with chosen domains if specified.
   1. If you want to choose by column and size 
      ```
      python3 ../src/homology_choose_domains.py Pfam-model.tsv -m "score" -n 100 --config_file config.yaml --new_config_file config-log-odd-100.yaml > homology_choose_domains.log
      ```
   2. If you want to choose with query
      ```
      python3 ../src/homology_choose_domains.py Pfam-model-no-pseudocount-fisher.tsv -q "fisher_score<0.05" --config_file config.yaml --new_config_file config-fisher-no-pseudocount.yaml
      ```

# Data availability 
Example of data can be found in data directory including files needed for homology. ESKAPEE data can be downloaded from plasgraph2-datasets [repository](https://github.com/fmfi-compbio/plasgraph2-datasets).


# Results summarization
You can also summarize 

1. Features

   ```
   python3 ../src/summarize_features.py ../sets/test_samples.csv  ../data/ test-pred.csv test-gold.csv -n pred,gold > test-summarized.csv'
   ```

2. Experiments - you can check example of experiments.csv 
   ```
   python3 src/summarize_experiments.py experiments.csv "" -o summarized-experiments.csv
   ```

3. Differences between two experiments:

   ```
   python3 ../src/summarize_differences.py experiment1/test-summarized.csv experiment2/test-summarized.csv --features_list length_norm coverage_norm gc_norm kmer_dot --names_for_experiments Pôvodné Homológia Spoločné Nesprávne --features_names "Relatívna dĺžka" "Relatívne pokrytie" "Relatívny obsah GC" "Relatívny obsah podreťazcov dĺžky k = 5" --translate_contigs_labels > summarize_differences.log'
   
   ```

# Tips 

You might need to use `export OMP_NUM_THREADS=1` for setting threads. 

